#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import os
import argparse
import logging

def hectar_transfer(hectar_file, orthofinder_file, gff_file, outdir):
    """
    Transfer of description between a manual annotation file and a gff file, using an orthofinder output file
    to find the corresponding IDs between the two files

    :param hectar_file:
    :param orthofinder_file:
    :param gff_file:
    :param outdir:
    :return:
    """

    manual_annotations_dict, orthofinder_dict, gff_dict = {}, {}, {}
    species_filename = ""

    with open(orthofinder_file, 'r') as orthofinder:
        next(orthofinder)  # Skip header
        # Mapping orthofinder's IDs to hectar IDs  
        logging.info("Mapping Orthofinder's IDs to hectar IDs...")
        for orthofiner_line in orthofinder:
            orthofiner_line_split = orthofiner_line.rstrip().split("\t")
            orthofinder_dict[orthofiner_line_split[1].split(" ")[0]] = orthofiner_line_split[2].split(" ")[0]
        
        hectar_dict = {}
        with open(hectar_file, 'r') as hectar:
            # Mapping orthofinder's IDs to hectar description
            logging.info("Mapping Orthofinder's IDs to descriptions...")
            next(hectar)  # Skip header
            for hectar_line in hectar:
                hectar_line_split = hectar_line.rstrip().split("\t")
                hectar_dict[hectar_line_split[0].split(" ")[0]] = hectar_line_split[1]

    # Opening GFF, appending hectar description to matching IDs
    logging.info("Transferring manual descriptions to the GFF file...")
    output_filename = str(os.path.join(os.path.abspath(outdir), os.path.basename(gff_file).split(".")[0]) + "_TRANSFERED_HECTAR.gff")

    with open(output_filename, 'a') as output_file:
        output_file.truncate(0)  # Erase previous file content
        with open(gff_file, 'r') as gff:
            for gff_line in gff:
                if "ID=mRNA" in gff_line:  # Look for mRNA items
                    gff_line_id = gff_line.split("\t")[8].split("=")[1].split(";")[0]  # GFF file ID (matches hectar IDs)
                    if gff_line_id in hectar_dict.keys():  # The gff ID is supposed to match a unique ID (key) in the orthofinder file
                        try:
                            hectar_value = hectar_dict[gff_line_id]  # Find the corresponding manual annotation to transfer
                            gff_line = "{0};HECTAR_predicted_targeting_category={1}\n".format(gff_line.strip(), hectar_value)
                        except KeyError:  # Just in case some values are missing in the hectar file (If removed it will then exit instead)
                            continue
                output_file.write(gff_line)
            logging.info("Finished transferring descriptions for %s" % str(os.path.basename(gff_file)))


if __name__ == "__main__":

    parser = argparse.ArgumentParser(description="Orthologs_transfer between a reference manual annotation and Orthofinder outputs. \
                                                  \n\nRun example: python3 ec32_orthologs_transfer.py -d /path/to/desired/dir/ -h hectar_file.tsv \
                                                  -g gff_file.gff -m manual_annotation_file.txt")
    parser.add_argument("-i", "--hectar",
                        help="Hectar output (tabulated)")
    parser.add_argument("-o", "--orthofinder",
                        help="Manual annotation file (tabulated) or pickle file containing the dicionary (automatically generated when you input a new manual annotation file)")
    parser.add_argument("-g", "--gff",
                        help="GFF input file (tabulated)")
    parser.add_argument("-d", "--out-dir",
                        help="Output directory")
    args = parser.parse_args()

    # WARNING for shell use case:  
    # The script doesn't check if the inout files are valid. 
    # Files must be put in this order: $1=orthofinder file, $2=manual annotation file, $3=GFF file, $4=optional output directory)

    if not args.hectar:
        logging.info("Please input a hectar file")
        sys.exit()
    if not args.orthofinder:
        logging.info("Please input a hectar output file")
        sys.exit()
    if not args.gff:
        logging.info("Please input a gff file")
        sys.exit()
    if not args.out_dir:
        args.out = "."

    hectar_transfer(hectar_file=args.hectar,
                       orthofinder_file=args.orthofinder,
                       gff_file=args.gff,
                       outdir=args.out_dir)
